import numpy as np

from ase import Atoms
from ase.io import read, write
from ase.lattice.surface import fcc111
from ase.lattice.surface import add_adsorbate
from ase.cluster.cubic import FaceCenteredCubic as FCCCluster
from ase.visualize import view


class ClusterError(ValueError):
    pass

msym = 'Pd'

a0values = dict(Ir=3.906,
                Rh=3.857,
                Au=4.218,
                W=4.078,
                Ru=3.856,
                Pd=3.988)

def build_supported_cluster(a0, msym, wcluster, hcluster, hcut):
    nnd = a0 / np.sqrt(2)

    system = fcc111(msym, a=a0, size=(8, 8, 5), orthogonal=True)
    tags = system.get_tags()
    base = system.positions[tags == 1][0]
    del system[tags == 1]



    cluster = FCCCluster('Pt',
                         layers=[hcluster + 1, wcluster],
                         surfaces=[[1., 1., 1.], [1., 0., 0.]],
                         latticeconstant=a0)
    cluster = Atoms(cluster.numbers, positions=cluster.positions,
                    cell=cluster.cell)

    dir3 = 1,1,1
    dir1 = -1,1,0
    dir2 = np.cross(dir3, dir1)

    A = np.array([np.array(dir) / np.linalg.norm(dir)
                  for dir in [dir1, dir2, dir3]])
    cluster.set_positions(np.dot(A, cluster.positions.T.copy()).T.copy())

    cen = cluster.positions.sum(axis=0) / len(cluster)

    zpos = cluster.positions[:, 2]
    zargs = np.argsort(zpos)
    cluster = cluster[zargs]
    zpos = cluster.positions[:, 2].copy()

    def group(sortednumbers, tolerance=0.1):
        args_li = []
        args_i = []
        val = sortednumbers[0]
        #print val
        for a, num in enumerate(sortednumbers):
            if num > val + tolerance:
                args_li.append(args_i)
                args_i = []
                val = num
            args_i.append(a)
        args_li.append(args_i)
        return args_li

    args_li = group(zpos)

    hplateau = hcluster - hcut + 1 # include "adsorbate layer"
    
    ngroups = len(args_li)
    assert (ngroups - 1) % 2 == 0
    nmaxgroups = (ngroups - 1) // 2
    delete_args = []
    #for args_i in args_li[:nmaxgroups]:
    for args_i in args_li[:-hplateau]:
        delete_args.extend(args_i)
    del cluster[delete_args]

    args_li = group(cluster.positions[:, 2])

    adsorbatesites = args_li[-1]
    cluster.numbers[adsorbatesites] = 8
    #print args_li, len(cluster)
    try:
        Ztop = cluster.positions[args_li[-2][0], 2]
    except IndexError:
        raise ClusterError('There won\'t be any Pt like this')

    d = 2.074
    dz = np.sqrt(d**2 - nnd**2 / 3.0)
    cluster.positions[adsorbatesites, 2] = Ztop + dz #1.35
    #cluster.positions[adsorbatesites, 2] = Ztop + 1.35
    xyref = cluster.positions[adsorbatesites].sum(axis=0) / len(adsorbatesites)
    #args_li = args_li[nmaxgroups:]

    if hplateau > nmaxgroups:
        raise ClusterError('Too many plateau layers for given cluster size')

    #print len(args_li)
    if len(args_li) != hplateau:
        raise ClusterError('Weird shape or something')

    assert sum([len(args_i) for args_i in args_li]) == len(cluster)

    #mask = np.vdot(pos - cen, [1., 1., 1,]) < 0
    #del cluster[np.vdot(pos - 


    #add_adsorbate(system, 'X', position='fcc', height=2.8)
    #fccpos = system.positions[-1]


    yargmin = np.argsort(cluster.positions[args_li[0], 1])[0]
    refpos = cluster.positions[yargmin]
    for D in [0, 1]:
        #system.positions[:, D] -= base[D]
        cluster.positions[:, D] -= refpos[D] - base[D]
    cluster.positions[:, 2] += base[2] - refpos[2]

    system += cluster

    system.center(vacuum=5.0, axis=2)
    #system.positions[:, 0] += 2.0 * a0
    #system.positions[:, 1] += a0
    system.positions[:, 2] -= 0.5

    nsites = len(adsorbatesites)
    center = system.positions[-nsites:].sum(axis=0) / nsites
    
    spos = system.get_scaled_positions()
    #for D in [0, 1]:
    #    spos[:, D] += 0.5 - center[D]
    
    system.set_scaled_positions(spos % 1.0)
    return system

gooddirs = set([x.split('.', 1)[1] for x in """O_Pt_Rh.1.1.00.003
O_Pt_Rh.1.1.00.010
O_Pt_Rh.1.2.00.004
O_Pt_Rh.1.2.00.012
O_Pt_Rh.1.3.00.013
O_Pt_Rh.2.1.00.001
O_Pt_Rh.2.1.00.007
O_Pt_Rh.2.1.02.001
O_Pt_Rh.2.1.03.007
O_Pt_Rh.2.1.04.007
O_Pt_Rh.3.1.00.005
O_Pt_Rh.3.1.04.005
O_Pt_Rh.3.1.06.005
O_Pt_Rh.3.2.00.006
O_Pt_Rh.3.2.03.006
O_Pt_Rh.3.2.04.006""".split()])

def isacceptable(dirname):
    return dirname.split('.', 1)[1] in gooddirs


a0 = a0values[msym]

def adjust(a0, msym, system):
    nnd = a0 / np.sqrt(2)
    system = system.copy()
    from ase.data import atomic_numbers
    system.numbers[system.numbers == 45] = atomic_numbers[msym]
    system.set_cell(system.get_cell() * a0 / 4.0, scale_atoms=True)

    Ztop = system.positions[:-1, 2].max()
    d = 2.074
    dz = np.sqrt(d**2 - nnd**2 / 3.0)
    system.positions[-1, 2] = Ztop + dz #1.35

    system.center(axis=2, vacuum=5.0)
    system.positions[:, 2] -= 0.5

    return system

def main(a0, msym, identifier='geom'):
    import os
    i = 0
    for h in range(1, 4):
        for w in range(1, 6):
            for N in range(h):
                try:
                    system = build_supported_cluster(4.0, 'Rh', h, w, N)
                except ClusterError:
                    continue

                NPt = len(system[system.numbers == 78])
                if NPt == 0:
                    continue
                
                i += 1
                Nsites = len(system[system.numbers == 8])


                basesystem = system[:-Nsites]
                assert len(basesystem) + Nsites == len(system)
                basesystem += Atoms('O')
                
                for Nsite in range(Nsites):
                    O = system.pop()
                    basesystem.positions[-1] = O.position
                    dirname = 'O_Pt_%s.%0d.%0d.%02d.%03d' % (msym,
                                                             h - N, w - N,
                                                             Nsite, i)
                    if not isacceptable(dirname):
                        continue
                    fname = '%s.traj' % identifier
                    path = os.path.join(dirname, fname)
                    print path
                    if not os.path.exists(dirname):
                        os.mkdir(dirname)
                    write(path, adjust(a0, msym, basesystem))

if __name__ == '__main__':
    for sym in a0values:
        main(a0values[sym], sym)
    #main(a0, msym)
